#!/usr/bin/env node

var minimist = require('minimist')
var hyperlog = require('hyperlog')
var level = require('level')
var path = require('path')
var exec = require('ssh-exec')
var mkdirp = require('mkdirp')

var argv = minimist(process.argv.slice(2), {alias:{merge:'m', path:'p'}})
var cmd = argv._[0]
var a = argv._[1]
var b = argv._[2]

mkdirp.sync(path.join(argv.path || '.', 'hyperlog'))
var log = hyperlog(level(path.join(argv.path || '.', 'hyperlog')))

if (cmd === 'sync' || cmd === 'pull' || cmd === 'push') {
  var input = process.stdin
  var output = process.stdout

  if (a && a.indexOf('@') > -1) {
    var p = a.split(':')[1]
    if (p && p.trim()) p = '--path '+p
    input = output = exec('hyperdb sync '+(p || ''), a.split(':')[0])
  } else if (a) {
  mkdirp.sync(path.join(a, 'hyperlog'))
    var remote = hyperlog(level(path.join(a, 'hyperlog'))).createReplicationStream()
    input = output = remote
  }

  input.pipe(log.createReplicationStream({mode:cmd})).pipe(output)
  return
}

if (cmd === 'get' && a) {
  var values = {}
  log.createChangesStream()
    .on('data', function(data) {
      var value = JSON.parse(data.value.toString())
      if (value.key === a) values[data.hash] = data
    })
    .on('end', function() {
      if (!argv.all) {      
        Object.keys(values).forEach(function(k) {
          var v = values[k]
          v.links.forEach(function(l) {
            delete values[l.hash]
          })
        })
      }
      Object.keys(values).forEach(function(k) {
        console.log(k+': '+JSON.parse(values[k].value.toString()).value)
      })
    })
  return
}

if (cmd === 'put' && a) {
  log.add(argv.merge, JSON.stringify({key:a, value:b}))
  return
}

console.log('Invalid command')
process.exit(1)